@@ -17,10 +17,11 @@ ImageFile.LOAD_TRUNCATED_IMAGES = True
 console = Console()
 
 # ============================================================
-# Core utilities
+# Core Utilities
 # ============================================================
 
 def progress_bar(task_name, total):
+    """Display a rich progress bar."""
     return Progress(
         SpinnerColumn(),
         TextColumn(f"[bold blue]{task_name}[/bold blue]"),
@@ -31,8 +32,8 @@ def progress_bar(task_name, total):
         transient=True,
     )
 
-
 def device_select(device="auto"):
+    """Auto-select device (cuda, mps, cpu)."""
     if device == "auto":
         if torch.cuda.is_available():
             return torch.device("cuda")
@@ -40,11 +41,10 @@ def device_select(device="auto"):
             return torch.device("mps")
         else:
             return torch.device("cpu")
-    else:
-        return torch.device(device)
-
+    return torch.device(device)
 
 def load_images(folder):
+    """Load all .jpg/.jpeg/.png images from a folder."""
     images, paths = [], []
     for f in os.listdir(folder):
         if f.lower().endswith((".jpg", ".jpeg", ".png")):
@@ -57,12 +57,22 @@ def load_images(folder):
                 continue
     return images, paths
 
-
 # ============================================================
-# CleanFrame main class
+# CleanFrame Class
 # ============================================================
 
 class CleanFrame:
+    """
+    CleanFrame:
+    A professional frame deduplication and clustering pipeline.
+
+    Features:
+      • Embeddings (ResNet, Swin, CLIP)
+      • Clustering (KMeans, DBSCAN)
+      • Deduplication via cosine similarity threshold
+      • Rich summary + notebook visualization
+    """
+
     def __init__(self, path, model="swin", cluster="kmeans", n_clusters=5,
                  device="auto", batch_size=32, eps=0.5, min_samples=5, threshold=0.95):
 
@@ -76,47 +86,58 @@ class CleanFrame:
         self.threshold = threshold
 
         self.device = device_select(device)
+        console.print(f"[bold cyan]Device:[/bold cyan] {self.device}")
+
         self.embeddings = None
         self.paths = None
         self.labels = None
         self.cleaned_paths = []
         self.removed_paths = []
 
+                # Create output folders next to original dataset
         name = os.path.basename(os.path.normpath(path))
-        self.root = os.path.join(".cleanframe_cache", name)
-        self.cleaned_dir = os.path.join(self.root, "cleaned")
-        os.makedirs(self.cleaned_dir, exist_ok=True)
+        base_dir = os.path.dirname(os.path.normpath(path))
 
-        console.print(f"[bold cyan]Device:[/bold cyan] {self.device}")
+        self.cleaned_dir = os.path.join(base_dir, f"{name}_cleaned")
+        self.removed_dir = os.path.join(base_dir, f"{name}_removed")
+
+        os.makedirs(self.cleaned_dir, exist_ok=True)
+        os.makedirs(self.removed_dir, exist_ok=True)
 
     # ============================================================
-    # Embedding extraction
+    # Embedding Extraction
     # ============================================================
 
     def embed_frames(self):
+        """Extract embeddings using the selected model."""
         images, paths = load_images(self.path)
         self.paths = paths
 
         if self.model_name == "resnet":
             model = models.resnet50(pretrained=True)
             model = nn.Sequential(*list(model.children())[:-1])
+            transform = transforms.Compose([
+                transforms.Resize((224, 224)),
+                transforms.ToTensor(),
+                transforms.Normalize(mean=[0.485, 0.456, 0.406],
+                                     std=[0.229, 0.224, 0.225])
+            ])
         elif self.model_name == "swin":
             model = create_model("swin_tiny_patch4_window7_224", pretrained=True, num_classes=0)
+            transform = transforms.Compose([
+                transforms.Resize((224, 224)),
+                transforms.ToTensor(),
+                transforms.Normalize(mean=[0.485, 0.456, 0.406],
+                                     std=[0.229, 0.224, 0.225])
+            ])
         elif self.model_name == "clip":
             model, _, preprocess = open_clip.create_model_and_transforms("ViT-B-32", pretrained="openai")
         else:
-            raise ValueError("model must be one of ['swin','clip','resnet'].")
+            raise ValueError("Model must be one of ['resnet', 'swin', 'clip'].")
 
         model.to(self.device).eval()
+        feats = []
 
-        tf = transforms.Compose([
-            transforms.Resize((224, 224)),
-            transforms.ToTensor(),
-            transforms.Normalize(mean=[0.485, 0.456, 0.406],
-                                 std=[0.229, 0.224, 0.225])
-        ])
-
-        embs = []
         with torch.inference_mode(), progress_bar(f"Embedding ({self.model_name})", len(images)) as p:
             task = p.add_task("Embedding", total=len(images))
             for i in range(0, len(images), self.batch_size):
@@ -126,15 +147,15 @@ class CleanFrame:
                     feat = model.encode_image(batch)
                     feat = feat / feat.norm(dim=-1, keepdim=True)
                 else:
-                    batch = torch.stack([tf(im) for im in batch_imgs]).to(self.device)
+                    batch = torch.stack([transform(im) for im in batch_imgs]).to(self.device)
                     feat = model(batch).squeeze()
                     if feat.ndim == 1:
                         feat = feat.unsqueeze(0)
                     feat = feat / feat.norm(dim=-1, keepdim=True)
-                embs.append(feat.cpu().numpy())
+                feats.append(feat.cpu().numpy())
                 p.update(task, advance=len(batch_imgs))
 
-        self.embeddings = np.concatenate(embs, axis=0)
+        self.embeddings = np.concatenate(feats, axis=0)
         return self.embeddings, self.paths
 
     # ============================================================
@@ -142,6 +163,7 @@ class CleanFrame:
     # ============================================================
 
     def cluster_frames(self):
+        """Cluster embeddings into groups of similar frames."""
         console.print(f"[cyan]Clustering using {self.cluster_method.upper()}...[/cyan]")
         emb = self.embeddings
         if self.cluster_method == "kmeans":
@@ -149,7 +171,7 @@ class CleanFrame:
         elif self.cluster_method == "dbscan":
             self.labels = DBSCAN(eps=self.eps, min_samples=self.min_samples).fit_predict(emb)
         else:
-            raise ValueError("cluster must be 'kmeans' or 'dbscan'")
+            raise ValueError("Cluster method must be 'kmeans' or 'dbscan'.")
         return self.labels
 
     # ============================================================
@@ -157,6 +179,7 @@ class CleanFrame:
     # ============================================================
 
     def clean_frames(self):
+        """Remove near-duplicate frames within each cluster."""
         console.print("[cyan]Cleaning frames...[/cyan]")
         emb, paths, labels = self.embeddings, self.paths, self.labels
         unique_clusters = [c for c in np.unique(labels) if c != -1]
@@ -189,16 +212,19 @@ class CleanFrame:
 
         self.cleaned_paths = kept
         self.removed_paths = removed
-        return kept, removed
+        console.print(f"[bold cyan]Kept:[/bold cyan] {len(kept)}   [bold red]Removed:[/bold red] {len(removed)}")
+        console.print(f"[yellow]Output → {self.cleaned_dir}[/yellow]")
 
     # ============================================================
     # Summary
     # ============================================================
 
-    def _summary_table(self):
+    def _summary(self):
+        """Display final pipeline summary table."""
         total = len(self.paths)
         kept = len(self.cleaned_paths)
         removed = len(self.removed_paths)
+
         table = Table(title="CLEANFRAMES SUMMARY", show_header=False, show_lines=False)
         table.add_row("Model", f"Swin Transformer" if self.model_name == "swin" else self.model_name.capitalize())
         table.add_row("Clustering", self.cluster_method.upper())
@@ -213,12 +239,17 @@ class CleanFrame:
         table.add_row("Report", f"{self.root}/report (use cf.report())")
         console.print(table)
 
+    # ============================================================
+    # Main pipeline
+    # ============================================================
+
     def run(self):
-        console.rule(f"[bold green]CLEANFRAME PIPELINE")
+        """Run the full deduplication pipeline."""
+        console.rule(f"[bold green]CLEANFRAME PIPELINE — {os.path.basename(self.path)}")
         self.embed_frames()
         self.cluster_frames()
         self.clean_frames()
-        self._summary_table()
+        self._summary()
         console.rule("[bold green]Done[/bold green]")
 
     # ============================================================
@@ -226,6 +257,7 @@ class CleanFrame:
     # ============================================================
 
     def report(self):
+        """Show detailed Rich + Matplotlib report."""
         console.print("[bold magenta]CleanFrames Detailed Report[/bold magenta]")
         console.print("---------------------------")
         console.print(f"Model: {self.model_name.capitalize()}")
@@ -234,7 +266,6 @@ class CleanFrame:
         console.print(f"Threshold: {self.threshold}")
         console.print("------------------------------------")
 
-        # pick 3 random pairs from all frames
         pairs = []
         if len(self.cleaned_paths) > 3:
             sample_idx = random.sample(range(len(self.cleaned_paths)), 6)
@@ -264,6 +295,7 @@ class CleanFrame:
         removed = len(self.removed_paths)
         kept_pct = (kept / total) * 100
         removed_pct = 100 - kept_pct
+
         console.print("Summary:")
         console.print(f"Cleaned with {self.model_name} embedding and {self.cluster_method}.")
         console.print(f"{kept_pct:.1f}% kept")
